package sim.portrayal.grid;
import sim.portrayal.*;
import sim.portrayal.simple.*;
import sim.field.grid.*;
import java.awt.*;
import java.awt.geom.*;
import sim.util.*;

/**
    A portrayal for grids containing objects, such as maybe agents or agent bodies.
*/

public class ObjectGrid2DPortrayal extends FieldPortrayal2D
    {
    public ObjectGrid2D field;
    
    // a grey oval.  You should provide your own protrayals...
    SimplePortrayal2D defaultPortrayal = new OvalPortrayal2D();
    
    public ObjectGrid2DPortrayal()
        {
        super();
        }
        
    public void setField(Object field)
        {
        if (field instanceof ObjectGrid2D ) this.field = (ObjectGrid2D) field;
        else throw new RuntimeException("Invalid field for ObjectGrid2DPortrayal: " + field);
        }
        
    public Object getField()
        {
        return field;
        }

    public Portrayal getDefaultPortrayal()
        {
        return defaultPortrayal;
        }

    protected void hitOrDraw(Graphics2D graphics, DrawInfo2D info, Bag putInHere)
        {
        if (field==null) return;
        if (field.field.length == 0) return;
        
        // Scale graphics to desired shape -- according to p. 90 of Java2D book,
        // this will change the line widths etc. as well.  Maybe that's not what we
        // want.
        
        // first question: determine the range in which we need to draw.
        // We assume that we will fill exactly the info.draw rectangle.
        // We can do the item below because we're an expensive operation ourselves
        
        int maxY = field.field[0].length; 
        int maxX = field.field.length;
        
        final double xScale = info.draw.width / maxX;
        final double yScale = info.draw.height / maxY;
        final int startx = (int)((info.clip.x - info.draw.x) / xScale);
        final int starty = (int)((info.clip.y - info.draw.y) / yScale); // assume that the X coordinate is proportional -- and yes, it's _width_
        int endx = /*startx +*/ (int)((info.clip.x - info.draw.x + info.clip.width) / xScale) + /*2*/ 1;  // with rounding, width be as much as 1 off
        int endy = /*starty +*/ (int)((info.clip.y - info.draw.y + info.clip.height) / yScale) + /*2*/ 1;  // with rounding, height be as much as 1 off

        DrawInfo2D newinfo = new DrawInfo2D(new Rectangle2D.Double(0,0, xScale, yScale),
                                            info.clip);  // we don't do further clipping 

        if (maxX>0 || maxY > 0)
            {
            if (endx > field.field.length) endx = field.field.length;
            for(int x=startx;x<endx;x++)
                {
                int tempendy = endy;  // keep endy around
                if (tempendy > field.field[x].length) tempendy = field.field[x].length;
                for(int y=starty;y<tempendy;y++)
                    {
                    Object obj = field.field[x][y];
                    Portrayal p = getPortrayalForObject(obj);
                    if (!(p instanceof SimplePortrayal2D))
                        throw new RuntimeException("Unexpected Portrayal " + p + " for object " + 
                                                    obj + " -- expected a SimplePortrayal2D");
                    SimplePortrayal2D portrayal = (SimplePortrayal2D)p;
                    
                    // translate --- the   + newinfo.width/2.0  etc. moves us to the center of the object
                    newinfo.draw.x = (int)(info.draw.x + (xScale) * x);
                    newinfo.draw.y = (int)(info.draw.y + (yScale) * y);
                    newinfo.draw.width = (int)(info.draw.x + (xScale) * (x+1)) - newinfo.draw.x;
                    newinfo.draw.height = (int)(info.draw.y + (yScale) * (y+1)) - newinfo.draw.y;
                    
                    // adjust drawX and drawY to center
                    newinfo.draw.x += newinfo.draw.width / 2.0;
                    newinfo.draw.y += newinfo.draw.height / 2.0;
                    
                    if (graphics == null)
                        {
                        if (portrayal.hitObject(obj, newinfo))
                            putInHere.add(getWrapper(new Int2D(x,y)));
                        }
                    else
                        portrayal.draw(obj, graphics, newinfo);
                    }
                }
            }
        }

    public LocationWrapper getWrapper(Int2D location)
        {
        return new LocationWrapper(null, location, this)
            {
            Int2D loc = (Int2D) this.location;
            public Object getObject()
                { 
                return field.field[loc.x][loc.y];
                }
            
            public String getLocationName()
                {
                return ((Int2D)this.location).toCoordinates();
                }
            };
        }
    }
